#lang racket

(require data/enumerate
         data/enumerate/lib)

;; write some test files

;; this enumerates nested arithmetic test cases:
(define nested-arith/e
  (delay/e
   (or/e (fin/e 1 2 87 143 7289)
         (list/e (single/e '-) nested-arith/e)
         (list/e (single/e '+) nested-arith/e nested-arith/e))))

;; this enumerates nested arithmetic test cases with read
(define nested-arith-read/e
  (delay/e
   (or/e (single/e '(read))
         (fin/e 1 2 87 143 7289)
         (list/e (single/e '-) nested-arith-read/e)
         (list/e (single/e '+) nested-arith-read/e nested-arith-read/e))))

;; this enumerates variable names
(define var/e (fin/e 'x 'y 'z 'a 'b 'c 'd))

;; this enumerates programs with let & nested arithmetic operations:
;; NB: most generated terms will have free vars, must patch manually
(define let/e
  (delay/e
   (or/e (fin/e 1 2 87 143 7289)
         var/e
         (list/e (single/e 'let)
                 (list/e (list/e var/e let/e))
                 let/e)
         (list/e (single/e '-) let/e)
         (list/e (single/e '+) let/e let/e)
         )))

;; this enumerates programs with let & read & nested arithmetic ops:
;; NB: most generated terms will have free vars, must patch manually
(define let-read/e
  (delay/e
   (or/e (fin/e 1 2 87 143 7289)
         var/e
         (single/e '(read))
         (list/e (single/e 'let)
                 (list/e (list/e var/e let-read/e))
                 let-read/e)
         (list/e (single/e '-) let-read/e)
         (list/e (single/e '+) let-read/e let-read/e)
         )))

;; here are some examples:
(from-nat nested-arith/e 22379418)
(from-nat nested-arith-read/e 7721497907)
(from-nat let/e 42938744423241231924123)
(from-nat let-read/e 272741982370129833729)

;; the library of tests. A few manual ones, then some random ones:
(define m1-test-cases
  '(("simple numbers"
     14)
    ("simple operators"
     (+ 3 4)
     (+ 15 9)
     (- 8))
    ("read"
     (read))
    ("nested arith"
     (+ (+ 3 8) 143)
     (+ (- (+ 4 2)) (+ (- 8) 9))
     ;; # 22379418:
     (+ (- (+ 7289 (- 2))) (- (+ (+ (+ 1 1) 2) (+ (+ 1 1) (- 1)))))
     )
    ("nested arith with read"
     (+ 3 (read))
     (+
      (- (- (+ (+ 1 (- (read))) (- (+ 1 2)))))
      (- (+ (- (- (- (+ 1 (read))))) (- (+ 2 (read)))))))
    ("let"
     (let ((x 4)) (+ x 4))
     (let ((x 4)) (let ((y 3)) (+ x y)))
     (let ((x (let ((z 9)) (+ z 1))))
       (let ([y (let ([w 4]) (- w))])
         (+ x y)))
     (+ (let ((x (+ 2 (let ((z 9)) (+ z 1)))))
          (let ([y (let ([w 4]) (- w))])
            (+ x y)))
        (let ((x 4)) (let ((y 3)) (+ x y))))
     (let ((z
            (let ((y (+ (let ((y 1)) (+ y (- 1))) (let ((z 1)) (- (+ z 1))))))
              (+ (let ((c 1)) (+ c y)) (let ((b 1)) (+ b 2))))))
       (+
        (- (let ((y z)) (+ (- 1) y)))
        (let ((y (- (let ((x 1)) x)))) (- (let ((b 1)) (+ b y))))))
     )
    ("let with read"
     (let ((x 4))
(+
  (let ((y (let ((a x)) (- (+ a (+ 1 x))))))
    (+ (let ((y 1)) (+ y x)) (- (- (- x)))))
  (let ((y (- (- (- (+ x (read)))))))
    (+ (let ((x 1)) (let ((x 41)) (+ y 1))) 143)))))))

(define test-case-path "/tmp/m1-test-cases")

;; write each test to a separate file:
(module+ main
  (for/list ([tc (in-list m1-test-cases)])
    (match-define (cons name exprs) tc)
    (define tc-stem (regexp-replace* #px" " name "-"))
    (for/list ([expr (in-list exprs)]
               [n (in-naturals)])
      (define target-path (build-path test-case-path
                                      (~a tc-stem "-" n ".sep")))
      (with-output-to-file target-path
        (λ () (pretty-write expr))))))